@@ -1,19 +1,20 @@
 class CacheTestMixin(object):
 
-    def make_cache(self, maxsize, getsizeof=None):
+    def cache(self, maxsize, getsizeof=None):
         raise NotImplementedError
 
-    def test_defaults(self):
-        cache = self.make_cache(maxsize=1)
+    def test_cache_defaults(self):
+        cache = self.cache(maxsize=1)
         self.assertEqual(0, len(cache))
         self.assertEqual(1, cache.maxsize)
         self.assertEqual(0, cache.currsize)
         self.assertEqual(1, cache.getsizeof(None))
         self.assertEqual(1, cache.getsizeof(''))
         self.assertEqual(1, cache.getsizeof(0))
+        self.assertTrue(repr(cache).startswith(cache.__class__.__name__))
 
-    def test_insert(self):
-        cache = self.make_cache(maxsize=2)
+    def test_cache_insert(self):
+        cache = self.cache(maxsize=2)
 
         cache.update({1: 1, 2: 2})
         self.assertEqual(2, len(cache))
@@ -30,8 +31,8 @@ class CacheTestMixin(object):
         self.assertEqual(4, cache[4])
         self.assertTrue(1 in cache or 2 in cache or 3 in cache)
 
-    def test_update(self):
-        cache = self.make_cache(maxsize=2)
+    def test_cache_update(self):
+        cache = self.cache(maxsize=2)
 
         cache.update({1: 1, 2: 2})
         self.assertEqual(2, len(cache))
@@ -48,8 +49,8 @@ class CacheTestMixin(object):
         self.assertEqual('a', cache[1])
         self.assertEqual('b', cache[2])
 
-    def test_delete(self):
-        cache = self.make_cache(maxsize=2)
+    def test_cache_delete(self):
+        cache = self.cache(maxsize=2)
 
         cache.update({1: 1, 2: 2})
         self.assertEqual(2, len(cache))
@@ -66,8 +67,8 @@ class CacheTestMixin(object):
         self.assertNotIn(1, cache)
         self.assertNotIn(2, cache)
 
-    def test_pop(self):
-        cache = self.make_cache(maxsize=2)
+    def test_cache_pop(self):
+        cache = self.cache(maxsize=2)
 
         cache.update({1: 1, 2: 2})
         self.assertEqual(2, cache.pop(2))
@@ -86,8 +87,8 @@ class CacheTestMixin(object):
         self.assertEqual(None, cache.pop(1, None))
         self.assertEqual(None, cache.pop(0, None))
 
-    def test_popitem(self):
-        cache = self.make_cache(maxsize=2)
+    def test_cache_popitem(self):
+        cache = self.cache(maxsize=2)
 
         cache.update({1: 1, 2: 2})
         self.assertIn(cache.pop(1), {1: 1, 2: 2})
@@ -98,8 +99,8 @@ class CacheTestMixin(object):
         with self.assertRaises(KeyError):
             cache.popitem()
 
-    def test_getsizeof(self):
-        cache = self.make_cache(maxsize=3, getsizeof=lambda x: x)
+    def test_cache_getsizeof(self):
+        cache = self.cache(maxsize=3, getsizeof=lambda x: x)
         self.assertEqual(3, cache.maxsize)
         self.assertEqual(0, cache.currsize)
         self.assertEqual(1, cache.getsizeof(1))
@@ -131,6 +132,12 @@ class CacheTestMixin(object):
         self.assertNotIn(1, cache)
         self.assertNotIn(2, cache)
 
+        with self.assertRaises(ValueError):
+            cache[3] = 4
+        self.assertEqual(1, len(cache))
+        self.assertEqual(3, cache.currsize)
+        self.assertEqual(3, cache[3])
+
         with self.assertRaises(ValueError):
             cache[4] = 4
         self.assertEqual(1, len(cache))
@@ -138,51 +145,32 @@ class CacheTestMixin(object):
         self.assertEqual(3, cache[3])
 
 
-class LRUCacheTestMixin(CacheTestMixin):
+class DecoratorTestMixin(object):
 
-    def test_lru_insert(self):
-        cache = self.make_cache(maxsize=2)
+    def decorator(self, maxsize, typed=False, lock=None):
+        raise NotImplementedError
 
-        cache[1] = 1
-        cache[2] = 2
-        cache[3] = 3
+    def test_decorator(self):
+        cached = self.decorator(maxsize=2)(lambda n: n)
 
-        self.assertEqual(len(cache), 2)
-        self.assertEqual(cache[2], 2)
-        self.assertEqual(cache[3], 3)
-        self.assertNotIn(1, cache)
+        self.assertEqual(cached(1), 1)
+        self.assertEqual(cached.cache_info(), (0, 1, 2, 1))
+        self.assertEqual(cached(1), 1)
+        self.assertEqual(cached.cache_info(), (1, 1, 2, 1))
+        self.assertEqual(cached(1.0), 1.0)
+        self.assertEqual(cached.cache_info(), (2, 1, 2, 1))
 
-        cache[2]
-        cache[4] = 4
-        self.assertEqual(len(cache), 2)
-        self.assertEqual(cache[2], 2)
-        self.assertEqual(cache[4], 4)
-        self.assertNotIn(3, cache)
+        cached.cache_clear()
+        self.assertEqual(cached(1), 1)
+        self.assertEqual(cached.cache_info(), (2, 2, 2, 1))
 
-        cache[5] = 5
-        self.assertEqual(len(cache), 2)
-        self.assertEqual(cache[4], 4)
-        self.assertEqual(cache[5], 5)
-        self.assertNotIn(2, cache)
-
-    def test_lru_getsizeof(self):
-        cache = self.make_cache(maxsize=3, getsizeof=lambda x: x)
-
-        cache[1] = 1
-        cache[2] = 2
-
-        self.assertEqual(len(cache), 2)
-        self.assertEqual(cache[1], 1)
-        self.assertEqual(cache[2], 2)
-
-        cache[3] = 3
-
-        self.assertEqual(len(cache), 1)
-        self.assertEqual(cache[3], 3)
-        self.assertNotIn(1, cache)
-        self.assertNotIn(2, cache)
-
-        with self.assertRaises(ValueError):
-            cache[4] = 4
-        self.assertEqual(len(cache), 1)
-        self.assertEqual(cache[3], 3)
+    def test_typed_decorator(self):
+        cached = self.decorator(maxsize=2, typed=True)(lambda n: n)
+        self.assertEqual(cached(1), 1)
+        self.assertEqual(cached.cache_info(), (0, 1, 2, 1))
+        self.assertEqual(cached(1), 1)
+        self.assertEqual(cached.cache_info(), (1, 1, 2, 1))
+        self.assertEqual(cached(1.0), 1.0)
+        self.assertEqual(cached.cache_info(), (1, 2, 2, 2))
+        self.assertEqual(cached(1.0), 1.0)
+        self.assertEqual(cached.cache_info(), (2, 2, 2, 2))
